[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# FactSet Fundamentals client library for Python

[![API Version](https://img.shields.io/badge/api-v2.2.0-blue)](https://developer.factset.com/api-catalog/factset-fundamentals-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.FactSetFundamentals)](https://pypi.org/project/fds.sdk.FactSetFundamentals/)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

Gain access to current, comprehensive, and comparative information on securities in worldwide developed and emerging markets. Composed of annual and interim/quarterly data, detailed historical financial statement content, per-share data, and calculated ratios, FactSet Fundamentals provides you with the information you need for a global investment perspective.<p>This API is rate-limited to 10 requests per second and 10 concurrent requests per user.</p>


This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 2.2.0
- SDK version: 2.3.2
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.FactSetFundamentals==2.3.2
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.FactSetFundamentals==2.3.2
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.FactSetFundamentals
from fds.sdk.FactSetFundamentals.api import batch_processing_api
from fds.sdk.FactSetFundamentals.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.FactSetFundamentals.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.FactSetFundamentals.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.FactSetFundamentals.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = batch_processing_api.BatchProcessingApi(api_client)
    id = "id_example" # str | Batch Request identifier.

    try:
        # Returns the response for a Batch Request 
        # example passing only required values which don't have defaults set
        api_response_wrapper = api_instance.get_batch_data(id)

        # This endpoint returns a response wrapper that contains different types of responses depending on the query.
        # To access the correct response type, you need to perform one additional step, as shown below.
        if api_response_wrapper.get_status_code() == 200:
            api_response = api_response_wrapper.get_response_200()
        if api_response_wrapper.get_status_code() == 202:
            api_response = api_response_wrapper.get_response_202()

        pprint(api_response)
    except fds.sdk.FactSetFundamentals.ApiException as e:
        print("Exception when calling BatchProcessingApi->get_batch_data: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Returns the response for a Batch Request 
    #     api_response_wrapper, http_status_code, response_headers = api_instance.get_batch_data_with_http_info(id)

    #     # This endpoint returns a response wrapper that contains different types of responses depending on the query.
    #     # To access the correct response type, you need to perform one additional step, as shown below.
    #     if api_response_wrapper.get_status_code() == 200:
    #         api_response = api_response_wrapper.get_response_200()
    #     if api_response_wrapper.get_status_code() == 202:
    #         api_response = api_response_wrapper.get_response_202()

    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetFundamentals.ApiException as e:
    #     print("Exception when calling BatchProcessingApi->get_batch_data: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Returns the response for a Batch Request 
    #     async_result = api_instance.get_batch_data_async(id)
    #     api_response_wrapper = async_result.get()

    #     # This endpoint returns a response wrapper that contains different types of responses depending on the query.
    #     # To access the correct response type, you need to perform one additional step, as shown below.
    #     if api_response_wrapper.get_status_code() == 200:
    #         api_response = api_response_wrapper.get_response_200()
    #     if api_response_wrapper.get_status_code() == 202:
    #         api_response = api_response_wrapper.get_response_202()

    #     pprint(api_response)
    # except fds.sdk.FactSetFundamentals.ApiException as e:
    #     print("Exception when calling BatchProcessingApi->get_batch_data: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Returns the response for a Batch Request 
    #     async_result = api_instance.get_batch_data_with_http_info_async(id)
    #     api_response_wrapper, http_status_code, response_headers = async_result.get()

    #     # This endpoint returns a response wrapper that contains different types of responses depending on the query.
    #     # To access the correct response type, you need to perform one additional step, as shown below.
    #     if api_response_wrapper.get_status_code() == 200:
    #         api_response = api_response_wrapper.get_response_200()
    #     if api_response_wrapper.get_status_code() == 202:
    #         api_response = api_response_wrapper.get_response_202()

    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.FactSetFundamentals.ApiException as e:
    #     print("Exception when calling BatchProcessingApi->get_batch_data: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.FactSetFundamentals

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.FactSetFundamentals.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.FactSetFundamentals

configuration = fds.sdk.FactSetFundamentals.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.FactSetFundamentals

configuration = fds.sdk.FactSetFundamentals.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.FactSetFundamentals

configuration = fds.sdk.FactSetFundamentals.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/content/factset-fundamentals/v2*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*BatchProcessingApi* | [**get_batch_data**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchProcessingApi.md#get_batch_data) | **GET** /batch-result | Returns the response for a Batch Request 
*BatchProcessingApi* | [**get_batch_status**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchProcessingApi.md#get_batch_status) | **GET** /batch-status | Returns the status for a Batch Request 
*CompanyReportsApi* | [**get_fds_profiles**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/CompanyReportsApi.md#get_fds_profiles) | **GET** /company-reports/profile | Returns company profile information for a specified list of identifiers.
*CompanyReportsApi* | [**get_fundamentals**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/CompanyReportsApi.md#get_fundamentals) | **GET** /company-reports/fundamentals | Returns company fundamentals for a given list of identifiers.
*FactSetFundamentalsApi* | [**get_fds_fundamentals_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FactSetFundamentalsApi.md#get_fds_fundamentals_for_list) | **POST** /fundamentals | Returns Company Fundamental Data.
*MetricsApi* | [**get_fds_fundamentals_metrics**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/MetricsApi.md#get_fds_fundamentals_metrics) | **GET** /metrics | Returns available FactSet Fundamental metrics and ratios.
*SegmentsApi* | [**get_fds_segments_for_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentsApi.md#get_fds_segments_for_list) | **POST** /segments | Returns Company Segment Data.


## Documentation For Models

 - [Address](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Address.md)
 - [Batch](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Batch.md)
 - [BatchErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchErrorObject.md)
 - [BatchResult](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchResult.md)
 - [BatchResultResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchResultResponse.md)
 - [BatchStatus](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchStatus.md)
 - [BatchStatusResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/BatchStatusResponse.md)
 - [CompanyFundamentalsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/CompanyFundamentalsResponse.md)
 - [CompanyReportErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/CompanyReportErrorObject.md)
 - [Dividend](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Dividend.md)
 - [ErrorObject](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/ErrorObject.md)
 - [ErrorObjectLinks](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/ErrorObjectLinks.md)
 - [ErrorResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/ErrorResponse.md)
 - [Exchange](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Exchange.md)
 - [FinancialResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FinancialResponse.md)
 - [Financials](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Financials.md)
 - [FiscalPeriod](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FiscalPeriod.md)
 - [Fundamental](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Fundamental.md)
 - [FundamentalFiscalPeriod](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FundamentalFiscalPeriod.md)
 - [FundamentalRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FundamentalRequestBody.md)
 - [FundamentalValue](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FundamentalValue.md)
 - [Fundamentals](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Fundamentals.md)
 - [FundamentalsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FundamentalsRequest.md)
 - [FundamentalsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/FundamentalsResponse.md)
 - [IdsBatchMax30000](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/IdsBatchMax30000.md)
 - [Item](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Item.md)
 - [Metric](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Metric.md)
 - [Metrics](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Metrics.md)
 - [MetricsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/MetricsResponse.md)
 - [Periodicity](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Periodicity.md)
 - [Profile](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Profile.md)
 - [ProfileResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/ProfileResponse.md)
 - [Segment](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/Segment.md)
 - [SegmentRequestBody](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentRequestBody.md)
 - [SegmentType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentType.md)
 - [SegmentValue](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentValue.md)
 - [SegmentsPeriodicity](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentsPeriodicity.md)
 - [SegmentsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentsRequest.md)
 - [SegmentsResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/SegmentsResponse.md)
 - [UpdateType](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/FactSetFundamentals/v2/docs/UpdateType.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.FactSetFundamentals.apis and fds.sdk.FactSetFundamentals.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.FactSetFundamentals.api.default_api import DefaultApi`
- `from fds.sdk.FactSetFundamentals.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.FactSetFundamentals
from fds.sdk.FactSetFundamentals.apis import *
from fds.sdk.FactSetFundamentals.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

